# frozen_string_literal: true

require 'open-uri'

class Reader
  attr_reader :genome

  def initialize(filename)
    @filename = filename
    @genome = GENOME::Genome.new @filename
  end

  def validate?(*)
    raise NotImplementedError, 'Reader must define validate?.'
  end

  def parse(*)
    raise NotImplementedError, 'Reader must parse validate?.'
  end

  def load
    validate? @filename
    open(@filename) do |f|
      f.each_line do |line|
        next if line.start_with?('#') || line.downcase.start_with?('rsid')

        rsid, snp = parse line
        @genome[rsid] = snp
      end
    end
  end
end

module GENOME
  class TwentyThreeAndMeReader < Reader
    def validate?(filename)
      magic_line = open(filename, &:readline)
      unless magic_line.downcase.include? '23'.downcase
        raise FileTypeError, 'format is not correct'
      end
    end

    def parse(line)
      rsid, chromosome, position, genotype = line.split
      snp = SNP.new(chromosome = chromosome,
                    position = position,
                    genotype = Genotype.new(genotype))
      [rsid, snp]
    end
  end
end

module GENOME
  class AncestryReader < Reader
    def validate?(filename)
      magic_line = open(filename, &:readline)
      unless magic_line.downcase.include? 'ancestry'.downcase
        raise FileTypeError, 'format is not correct'
      end
    end

    def parse(line)
      rsid, chromosome, position, allele1, allele2 = line.split
      genotype = allele1 + allele2
      snp = SNP.new(chromosome = chromosome,
                    position = position,
                    genotype = Genotype.new(genotype))
      [rsid, snp]
    end
  end
end

module GENOME
  class MyHeritageReader < Reader
    def validate?(filename)
      magic_line = open(filename, &:readline)
      unless magic_line.downcase.include? 'MyHeritage'.downcase
        raise FileTypeError, 'format is not correct'
      end
    end

    def parse(line)
      rsid, chromosome, position, genotype = line.gsub(/"/, '').gsub("\n", '').split(',')
      snp = SNP.new(chromosome = chromosome,
                    position = position,
                    genotype = Genotype.new(genotype))
      [rsid, snp]
    end
  end
end
